package de.assono;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.LinkedList;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import lotus.domino.NotesException;
import lotus.domino.NotesFactory;
import lotus.domino.NotesThread;
import lotus.domino.Session;

import org.gnu.stealthp.rsslib.RSSChannel;
import org.gnu.stealthp.rsslib.RSSException;
import org.gnu.stealthp.rsslib.RSSHandler;
import org.gnu.stealthp.rsslib.RSSItem;
import org.gnu.stealthp.rsslib.RSSParser;

public class DominoHelperServlet extends HttpServlet {

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		handleRequest(request, response);
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		handleRequest(request, response);
	}

	void handleRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		response.setContentType("text/html");
		PrintWriter writer = response.getWriter();
		writer.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" ");
		writer.println("\"http://www.w3.org/TR/html4/loose.dtd\">");
		writer.println("<html><head><title>Domino-Helper-Servlet</title></head><body>");

		writer.println("<p><b>" + getQotD() + "</b></p>");

		String requestURI = request.getRequestURI();
		writer.println("<!-- requestURI: " + requestURI + " -->");
		String action = request.getParameter("action");
		writer.println("<!-- action: " + action + " -->\n");

		addActionButton(writer, requestURI, "showStats", "Show HTTP related statistics");
		addActionButton(writer, requestURI, "clearCaches", "Clear command and design caches");
		addActionButton(writer, requestURI, "refresh", "Refresh HTTP configuration");
		addActionButton(writer, requestURI, "restartHTTP", "Restart HTTP task");
		addActionButton(writer, requestURI, "restartServer", "Restart Domino Server");

		if ("".equals(action)) {
			writer.println("</body></html>");
			writer.close();

		} else {
			try {
				NotesThread.sinitThread();
				Session session = NotesFactory.createSession();

				if ("showStats".equals(action)) {
					writer.println("<h2>HTTP statistics</h2>");
					writer.println("<pre>" + session.sendConsoleCommand(session.getServerName(), "show stat http.*") + "</pre>");
				}

				writer.println("</body></html>");
				writer.close();

				if ("clearCaches".equals(action)) {
					session.sendConsoleCommand(session.getServerName(), "tell http clearcaches");
				} else if ("refresh".equals(action)) {
					session.sendConsoleCommand(session.getServerName(), "tell http refresh");
				} else if ("restartHTTP".equals(action)) {
					session.sendConsoleCommand(session.getServerName(), "tell http restart");
				} else if ("restartServer".equals(action)) {
					session.sendConsoleCommand(session.getServerName(), "restart server");
				}

				session.recycle();
				session = null;
				NotesThread.stermThread();

			} catch (NotesException ne) {
				System.out.println("Notes-Fehler #" + ne.id + ": " + ne.text);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private void addActionButton(PrintWriter writer, String requestURI, String action, String label) {
		writer.println("<form action=\"" + requestURI + "?action=" + action + "\" method=\"post\">");
		writer.println("<input type=\"submit\" value=\"" + label + "\">");
		writer.println("</form>\n");
	}

	private String getQotD() {
		final String QOTD_FEED_URL = "http://feeds.feedburner.com/quotationspage/qotd";

		String quote = "";

		RSSHandler handler = new RSSHandler();
		try {
			RSSParser.parseXmlFile(new URL(QOTD_FEED_URL), handler, false);
			RSSChannel channel = handler.getRSSChannel();
			LinkedList list = channel.getItems();
			int randomPos = (int) Math.floor(Math.random() * list.size());
			RSSItem itm = (RSSItem) list.get(randomPos);
			String desc = itm.getDescription();
			quote = desc.substring(0, desc.indexOf("<p>")) + ", <i>" + itm.getTitle() + "</i>\n";
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (RSSException e) {
			e.printStackTrace();
		}

		return quote;
	}
}
